defmodule OpWeb.QueriLive.FormComponent do
  use OpWeb, :live_component

  alias Op.Base

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        <%= @title %>
        <:subtitle>Use this form to manage queri records in your database.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="queri-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:label]} type="text" label="Label" />
        <.input field={@form[:source]} type="text" label="Source" />
        <.input field={@form[:code]} type="text" label="Code" />
        <.input field={@form[:lang]} type="text" label="Lang" />
        <.input field={@form[:applies]} type="datetime-local" label="Applies" />
        <.input field={@form[:expires]} type="datetime-local" label="Expires" />
        <.input field={@form[:cache]} type="text" label="Cache" />
        <.input field={@form[:chronology]} type="text" label="Chronology" />
        <.input field={@form[:signed]} type="text" label="Signed" />
        <:actions>
          <.button phx-disable-with="Saving...">Save Queri</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{queri: queri} = assigns, socket) do
    changeset = Base.change_queri(queri)

    {:ok,
     socket
     |> assign(assigns)
     |> assign_form(changeset)}
  end

  @impl true
  def handle_event("validate", %{"queri" => queri_params}, socket) do
    changeset =
      socket.assigns.queri
      |> Base.change_queri(queri_params)
      |> Map.put(:action, :validate)

    {:noreply, assign_form(socket, changeset)}
  end

  def handle_event("save", %{"queri" => queri_params}, socket) do
    save_queri(socket, socket.assigns.action, queri_params)
  end

  defp save_queri(socket, :edit, queri_params) do
    case Base.update_queri(socket.assigns.queri, queri_params) do
      {:ok, queri} ->
        notify_parent({:saved, queri})

        {:noreply,
         socket
         |> put_flash(:info, "Queri updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp save_queri(socket, :new, queri_params) do
    case Base.create_queri(queri_params) do
      {:ok, queri} ->
        notify_parent({:saved, queri})

        {:noreply,
         socket
         |> put_flash(:info, "Queri created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign_form(socket, changeset)}
    end
  end

  defp assign_form(socket, %Ecto.Changeset{} = changeset) do
    assign(socket, :form, to_form(changeset))
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})
end
